/*===========================================================================*/
/*
 * This file is part of
 * libneuro - a C++ library for signals in the Neuro format
 *
 * Copyright (C) 2006, 2007, 2008  Elaine Tsiang YueLien
 *
 * libneuro is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, see http://www.gnu.org/licenses, or write
 * Free Software Foundation, Inc.
 * 51 Franklin Street, Fifth Floor
 * Boston, MA  02110-1301, USA
 *
 *===========================================================================*/
/** @file neuro2raw.C                                                        */
/*                                                                           */
/* neuro2raw - convert Neuro format file to raw signal                       */
/*                                                                           */
/*===========================================================================*/
#include	<Neuro/Neuro.H>
#include	<Thread.H>
#include	<Ogg/Debug.H>

#include	<fstream>

extern "C" 
{
#include	<string.h>
#include	<unistd.h>
}

namespace	Examples
{
  using namespace	Ogg;
  using namespace	ALingA;
  using namespace	Neuro;

  class Demuxer: public Transport, public Thread
  {
  public:
    Demuxer()
      : Transport(false)
    {}

    void
    run()
    {
      loop();
    }

    size_t
    recvCallback(
		 Page &	pg
		 )
    {
      size_t count = Transport::recvCallback(pg);

      if ( std::cin.eof()
	   ||
	   (count <= 0 )
	   )
	{
	  terminate();
	}

      return(count);
    }
  }
  ;

  class NeuroParser : public ::Neuro::Neuro, public Thread
  {
  public:
    NeuroParser(
		Demuxer &	demuxer
		)
      : Neuro(demuxer)
    {}


    void
    run()
    {
      try
	{
	  iFrames & i = NeuroParser::read();

	  // print header info
	  ALingA::Manifold & manifold = NeuroParser::manifold();
	  
	  debug(true,
		"Track ", track()
		);

	  Attributes attributes(manifold.attributes());

	  for ( Attributes::const_iterator ai = attributes.begin()
		  ;
		ai != attributes.end()
		  ;
		++ai
		)
	    {
	      debug(true
		    ,ai->name.c_str()
		    ," = "
		    ,ai->value.c_str()
		    );
	    }

	  do
	    {
	      if ( 0 == i.frameNo() % framesPerPacket() )
		debug(true,
		      " writing ",i.frameNo(),"th frame, "
		      );
	      
	      // write out frames as received
	      std::cout.write(reinterpret_cast<char *>(i.frame())
			      ,i.frameSize()
			      );

	      ++i;
	    }
	  while ( !i.ended() );

	  debug(true,
		" wrote last ",i.frameNo(),"th frame, "
		);
	      
	}
      catch ( exception ex
	      )
	{
	  debug(true,
		ex.what()
		);
	  throw;
	}

      return;
    }
      
  }
  ;

}

int main(int argc, const char* argv[])
{
  using namespace	Examples;


  Demuxer * transport = new Demuxer;
  NeuroParser * neuroParser = new NeuroParser(*transport);

  transport->start();
  neuroParser->start();

  neuroParser->finish();
  transport->finish();

  exit(0);
}
